package Local::DBD::ADO::DSN;
[
  {
    'Provider'    => 'Microsoft.Jet.OLEDB.4.0'
  , 'Data Source' => '/data/test.mdb'
  }
, {
    'Provider'    => 'VFPOLEDB'
  , 'Data Source' => 'C:/data/test.dbc'
  }
, {
    'Provider'    => 'MSDAORA'
  , 'Data Source' => ''
  , 'User ID'     => 'scott'
  , 'Password'    => 'tiger'
  }
, {
    'File Name'   => '/data/test.udl'
  }
]
